@@ -106,6 +106,70 @@ BENCHMARK_INFO = {
 
 class PortfolioDataApi(metaclass=Singleton):
 
+    def get_all_fund_info(self, fund_list=None):
+        try:
+            if fund_list is not None:
+                mutual_fund_list = [_ for _ in fund_list if '!' in _]
+                hedge_fund_list = [_ for _ in fund_list if '!' not in _]
+                mutual_fund_info = BasicDataApi().get_fund_info(fund_list=mutual_fund_list)[['fund_id','desc_name']]
+                hedge_fund_info = RawDataApi().get_hf_fund_info(fund_list=hedge_fund_list)[['fund_id','desc_name']]
+            else:
+                mutual_fund_info = BasicDataApi().get_fund_info()[['fund_id','desc_name']]
+                hedge_fund_info = RawDataApi().get_hf_fund_info()[['fund_id','desc_name']]
+            if not mutual_fund_info.empty:
+                mutual_fund_info.loc[:,'fund_type'] = '公募'    
+            if not hedge_fund_info.empty:
+                hedge_fund_info.loc[:,'fund_type'] = '私募'
+            df_list = [i for i in [mutual_fund_info, hedge_fund_info] if not i.empty ]
+            fund_info = pd.concat(df_list)
+            return fund_info
+
+        except Exception as e:
+            print(f'Failed to get data <err_msg> {e} from portfolio.get_all_fund_info')
+
+    def get_part_fund_info(self, fund_list=None):
+        try:
+            mutual_fund_list = [_ for _ in fund_list if '!' in _]
+            hedge_fund_list = [_ for _ in fund_list if '!' not in _]
+            hedge_fund_info = pd.DataFrame()
+            mutual_fund_info = pd.DataFrame()
+            if len(mutual_fund_list) > 0:
+                mutual_fund_info = BasicDataApi().get_fund_info(fund_list=mutual_fund_list)[['fund_id','desc_name']]
+                mutual_fund_info.loc[:,'fund_type'] = '公募'    
+            if len(hedge_fund_list) > 0:
+                hedge_fund_info = RawDataApi().get_hf_fund_info(fund_list=hedge_fund_list)[['fund_id','desc_name']]
+                hedge_fund_info.loc[:,'fund_type'] = '私募'
+            df_list = [i for i in [mutual_fund_info, hedge_fund_info] if not i.empty ]
+            fund_info = pd.concat(df_list)
+            return fund_info
+
+        except Exception as e:
+            print(f'Failed to get data <err_msg> {e} from portfolio.get_part_fund_info')
+
+
+    def get_all_fund_nav(self, start_date='', end_date='', fund_list: Tuple[str] = ()):
+        try:
+            if fund_list is not None:
+                mutual_fund_list = [_ for _ in fund_list if '!' in _]
+                hedge_fund_list = [_ for _ in fund_list if '!' not in _]
+                mutual_fund_nav = BasicDataApi().get_fund_nav_with_date(start_date=start_date-datetime.timedelta(days=20),end_date=end_date,fund_list=fund_list)
+                hedge_fund_nav = RawDataApi().get_hf_fund_nav(fund_ids=hedge_fund_list, start_date=start_date-datetime.timedelta(days=20), end_date=end_date)
+            else:
+                mutual_fund_nav = BasicDataApi().get_fund_nav_with_date(start_date=start_date-datetime.timedelta(days=20),end_date=end_date)
+                hedge_fund_nav = RawDataApi().get_hf_fund_nav_dt(start_date=start_date-datetime.timedelta(days=20))
+            
+            if not mutual_fund_nav.empty:
+                mutual_fund_nav = mutual_fund_nav.pivot_table(index='datetime',columns='fund_id',values='adjusted_net_value')
+            if not hedge_fund_nav.empty:
+                hedge_fund_nav = hedge_fund_nav.pivot_table(index='datetime',columns='fund_id',values='nav')
+            df_list = [i for i in [mutual_fund_nav, hedge_fund_nav] if not i.empty ]
+            fund_nav = pd.concat(df_list, axis=1).sort_index().ffill().loc[start_date:]
+            fund_nav.index.name = 'datetime'
+            return fund_nav
+
+        except Exception as e:
+            print(f'Failed to get data <err_msg> {e} from portfolio.get_all_fund_nav')
+
     def trade_history_parse(self, trade_history):
         try:
             fund_type = {}
@@ -135,8 +199,17 @@ class PortfolioDataApi(metaclass=Singleton):
             order_list, fund_type = self.trade_history_parse(trade_history)
             fund_list = set(list([j for dic in order_list for i in dic.values() for j in i.keys()]))
             _begin_date = min([j for i in order_list for j in i.keys()])
-            fund_nav = BasicDataApi().get_fund_nav_with_date(start_date=_begin_date,fund_list=fund_list)
-            fund_nav = fund_nav.pivot_table(index="datetime" , values="adjusted_net_value" , columns="fund_id").dropna()
+            fund_nav = self.get_all_fund_nav(start_date=_begin_date,fund_list=fund_list)
+            if fund_nav.empty:
+                print('not fund nav existed')
+                return {
+                    '净值':pd.DataFrame(),
+                    '各基净值':pd.DataFrame(),
+                    '各基权重':pd.DataFrame(),
+                    '大类净值':pd.DataFrame(),
+                    '持有信息':{},
+                }
+            fund_nav = fund_nav.ffill()
             order_dic = { dt: i[dt] for i in order_list for dt in list(i.keys())}
             date_list = [j for i in order_list for j in i.keys()] + [fund_nav.index[-1]]
             date_pairs = [[dt, date_list[idx+1]] for idx, dt in enumerate(date_list) if dt != date_list[-1]]
@@ -169,6 +242,7 @@ class PortfolioDataApi(metaclass=Singleton):
                 port_type_result.append(fund_type_amt_dt)
             port_mv_result = [i.iloc[:-1] if i.index[-1] != port_mv_result[-1].index[-1] else i for i in port_mv_result]
             port_mv_result = pd.concat(port_mv_result)
+            port_mv_result.index.name = 'datetime'
             port_weight_result = [i.iloc[:-1] if i.index[-1] != port_weight_result[-1].index[-1] else i for i in port_weight_result]
             port_weight_result = pd.concat(port_weight_result)
             port_type_result = [i.iloc[:-1] if i.index[-1] != port_type_result[-1].index[-1] else i for i in port_type_result]
@@ -197,6 +271,15 @@ class PortfolioDataApi(metaclass=Singleton):
 
     def get_portfolio_mv(self, fund_nav, index_id, begin_date, end_date, time_para):
         try:
+            if fund_nav.empty:
+                df = pd.DataFrame()
+                df.index.name = 'datetime'
+                data = {
+                    'data':df,
+                    'mdd_1':{},
+                    'stats':[],
+                }
+                return data
             def print_st(mdd):
                 return str(round(mdd * 100,2)) + '%' 
             fund_nav.columns=['组合']
@@ -228,7 +311,6 @@ class PortfolioDataApi(metaclass=Singleton):
             st1 = f'年化收益: {ar} 年化波动: {av} 夏普比率: {sharpe}'
             st2 = f'最大回撤 {mdd1} {mdd1_d1} : {mdd1_d2}, {mdd1_lens} days'
             st3 = f'第二回撤 {mdd2} {mdd2_d1} : {mdd2_d2}, {mdd2_lens} days'
-            sts = [st1, st2, st3]
             mdd_dic_1 = {
                 'd1':str(mdd_details['mdd1_d1']),
                 'd2':str(mdd_details['mdd1_d2']),
@@ -242,22 +324,31 @@ class PortfolioDataApi(metaclass=Singleton):
                 'v2':df['组合'].loc[mdd_details['mdd2_d1']: mdd_details['mdd2_d2']].max(),
             }
             data = {
-                'data':df,
-                'mdd_1':mdd_dic_1,
-                'mdd_2':mdd_dic_2,
-                'stats': sts
-            }
+                    'data':df,
+                    'mdd_1':mdd_dic_1,
+                    'stats': [st1, st2]
+                }
+            if mdd_dic_2['d1'] != 'None':
+                data['mdd_2'] = mdd_dic_2
+                data['stats'].append(st3)
             return data
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_portfolio_mv')
 
     def get_portfolio_cur_mdd(self, fund_nav, begin_date, end_date, time_para):
-        try:            
+        try:        
+            if fund_nav.empty:
+                data = {
+                    '历史回撤':pd.DataFrame(),
+                    '最低值':None,
+                }    
+                return data
             begin_date, end_date = RawDataApi().get_date_range(time_para, begin_date, end_date)
             fund_nav = fund_nav.loc[begin_date:end_date]
             df = fund_nav / fund_nav.cummax() - 1
             df.columns=['历史回撤']
             df_min = df['历史回撤'].min()
+            df.index.name = 'datetime'
             data = {
                 '历史回撤': df,
                 '最低值': df_min,
@@ -268,6 +359,13 @@ class PortfolioDataApi(metaclass=Singleton):
 
     def get_portfolio_ret_distribution(self, fund_nav, period, begin_date, end_date, time_para):
         try:
+            if fund_nav.empty:
+                data = {
+                    '分布_频度':[],
+                    '分布_收益':[],
+                    '收益':pd.DataFrame(),
+                }
+                return data
             begin_date, end_date = RawDataApi().get_date_range(time_para, begin_date, end_date)
             fund_nav = fund_nav.loc[begin_date:end_date]    
             fund_nav = CalculatorBase.data_resample_weekly_nav(df=fund_nav, rule=period)
@@ -287,6 +385,9 @@ class PortfolioDataApi(metaclass=Singleton):
 
     def get_portfolio_stats(self, fund_nav, index_id):
         try:
+            if fund_nav.empty:
+                return pd.DataFrame()
+
             fund_nav.columns=['组合']
             basic = BasicDataApi()
             index_price = basic.get_index_price_dt(index_list=[index_id],start_date=fund_nav.index[0] - datetime.timedelta(days=10),end_date=fund_nav.index[-1]).pivot_table(index='datetime',columns='index_id',values='close')
@@ -407,26 +508,37 @@ class PortfolioDataApi(metaclass=Singleton):
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_rolling_corr')
     
-    def get_port_fund_list(self, order_list):
+    def get_port_fund_list(self, trade_history):
         try:
+            order_list, fund_type = self.trade_history_parse(trade_history)
             fund_list = []
             for order in order_list:
                 for d, fund_wgt in order.items():
                     fund_list.extend(list(fund_wgt))
-            return list(set(fund_list))
+            fund_list = list(set(fund_list))
+            fund_info = self.get_all_fund_info(fund_list)
+            result = [[r.fund_id, r.desc_name] for r in fund_info.itertuples()]
+            return result
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_port_fund_list')
 
-    def get_port_fund_pos_date(self,fund_id, order_list,index_id): 
+    def get_port_fund_pos_date(self,fund_id, trade_history, index_id): 
         try:
+            order_list, fund_type = self.trade_history_parse(trade_history)
             basic = BasicDataApi()
             fund_list = [fund_id]
-            fund_info = basic.get_fund_info(fund_list=fund_list)
+            fund_info = self.get_part_fund_info(fund_list=fund_list)
             desc_name_dic = fund_info.set_index('fund_id').to_dict()['desc_name']
 
             begin_date, end_date = RawDataApi().get_date_range(time_para='ALL', begin_date=None, end_date=None)
-            fund_nav = basic.get_fund_nav(fund_list=[fund_id])
-            fund_nav = fund_nav.pivot_table(columns='fund_id',index='datetime',values='adjusted_net_value')
+            fund_nav = self.get_all_fund_nav(fund_list=[fund_id])
+            if fund_nav.empty:
+                data = {
+                '净值':pd.DataFrame(),
+                '持有期':[],
+                }
+                return data
+            fund_nav = fund_nav.ffill()
             index_price = basic.get_index_price(index_list=[index_id])
             index_price = index_price.pivot_table(columns='index_id',index='datetime',values='close')
             nav_df = fund_nav.join(index_price).ffill().dropna()
@@ -467,9 +579,9 @@ class PortfolioDataApi(metaclass=Singleton):
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_port_fund_pos_date')
 
-    def get_port_ret_resolve_date_range(self, order_list):
+    def get_port_ret_resolve_date_range(self, trade_history):
         try:
-            
+            order_list, fund_type = self.trade_history_parse(trade_history)
             begin_date, end_date = RawDataApi().get_date_range(time_para='ALL', begin_date=None, end_date=None)
             begin_date = list(order_list[0].keys())[0]
             data = {'begin_date':begin_date,'end_date':end_date}
@@ -479,9 +591,11 @@ class PortfolioDataApi(metaclass=Singleton):
     
     def get_port_ret_resolve(self, share_list , begin_date , end_date):
         try:
+            if len(share_list) < 1:
+                return []
             basic = BasicDataApi()
             fund_list = list(set([i for dt, share_list in share_list.items() for i in list(share_list.keys())]))
-            fund_info = basic.get_fund_info(fund_list=fund_list)
+            fund_info = self.get_all_fund_info(fund_list=fund_list)
             desc_name_dic = fund_info.set_index('fund_id').to_dict()['desc_name']
             all_ret_dict = []
             for i in range(len(list(share_list.keys()))):
@@ -507,8 +621,8 @@ class PortfolioDataApi(metaclass=Singleton):
                 ret_dict = {}
                 for fund_id in list(share_list[this_date].keys()):
                     fund_share = share_list[this_date][fund_id]
-                    fund_nav = basic.get_fund_nav_with_date(start_date=cal_start,end_date=cal_end,fund_list=[fund_id])
-                    fund_nav = fund_nav.pivot_table(index="datetime" , values="adjusted_net_value" , columns="fund_id").dropna()
+                    fund_nav = self.get_all_fund_nav(start_date=cal_start,end_date=cal_end,fund_list=[fund_id])
+                    fund_nav = fund_nav.ffill()
                     fund_ret = round((fund_nav.iloc[-1].values[0] * fund_share - fund_nav.iloc[0].values[0] * fund_share)*100,2)
                     ret_dict[desc_name_dic[fund_id]] = fund_ret
                 all_ret_dict.append(ret_dict)
@@ -518,8 +632,16 @@ class PortfolioDataApi(metaclass=Singleton):
 
     def get_port_fund_weight(self, fund_wgt, trade_history):
         try:
+            if fund_wgt.empty:
+                data = {
+                    '基金':pd.DataFrame(),
+                    '类型':pd.DataFrame(),
+                    '类型对照':{},
+                }
+                return data
+
             order_list, fund_type = self.trade_history_parse(trade_history)
-            fund_info = BasicDataApi().get_fund_info(fund_wgt.columns.tolist())
+            fund_info = self.get_all_fund_info(fund_wgt.columns.tolist())
             name_dic = fund_info.set_index('fund_id').to_dict()['desc_name']
             fund_wgt = fund_wgt.rename(columns=name_dic)
             result = {}
@@ -536,7 +658,6 @@ class PortfolioDataApi(metaclass=Singleton):
             dts = fund_wgt.index.tolist()
             dts = [dt for idx, dt in enumerate(dts) if idx % 10 == 0]
             fund_wgt_part_1 = fund_wgt.loc[dts]
-            wgt_index_list = fund_wgt_after.index.tolist()
             index_list = trade_list + [fund_wgt.index[0]] + [fund_wgt.index[-1]]
             fund_wgt_part_2 = fund_wgt.loc[fund_wgt.index.intersection(index_list)]
             fund_wgt = pd.concat([fund_wgt_part_1,fund_wgt_part_2]).sort_index()
@@ -554,6 +675,13 @@ class PortfolioDataApi(metaclass=Singleton):
             order_list, fund_type = self.trade_history_parse(trade_history)
             result = self.get_portfolio_nav(trade_history=trade_history)
             df = result['净值']
+            if data.empty:
+                data = {
+                    'mdd': pd.DataFrame(),
+                    'fund_nav': pd.DataFrame(),
+                }
+                return data
+
             fund_wgt = result['各基权重']
             fund_nav = result['各基净值']
 
@@ -574,23 +702,27 @@ class PortfolioDataApi(metaclass=Singleton):
             else:
                 l = [mdd_date1] + exchange_dts + [mdd_date2]
                 date_list = [ [i,l[l.index(i)+1]] for i in l if i != l[-1]]
-            fund_info = BasicDataApi().get_fund_info(fund_list=fund_list)
+            fund_info = self.get_all_fund_info(fund_list=fund_list)
             desc_name_dic = fund_info.set_index('fund_id').to_dict()['desc_name']
-
+            _mdd_list = []
             for date_group in date_list:
                 b_d = date_group[0]
                 e_d = date_group[1]
                 fund_wgt_dt = fund_wgt.loc[b_d].dropna()
-                mdd_i = ((fund_wgt.loc[e_d] / fund_wgt.loc[b_d] - 1) * fund_wgt_dt).to_dict()
-                break
+                mdd_i = ((fund_nav.loc[e_d] / fund_nav.loc[b_d] - 1) * fund_wgt_dt)
+                _mdd_list.append(mdd_i)
+            mdd_dict = ((pd.concat(_mdd_list,axis=1).fillna(0) + 1).prod(axis=1) - 1).to_dict()
             df_result = pd.DataFrame([desc_name_dic]).T
             df_result.columns = ['基金名']
-            df_result.loc[:,'回撤贡献'] = df_result.index.map(mdd_i)
+            df_result.loc[:,'回撤贡献'] = df_result.index.map(mdd_dict)
             df_result.loc[:,'类型'] = df_result.index.map(fund_type_dic)
             df_result = df_result.dropna()
             df_result.index.name = '基金代码'
-            df_result = df_result.reset_index()    
-            fund_nav = fund_nav.loc[mdd_date1:mdd_date2].rename(columns=desc_name_dic)  
+            df_result = df_result.reset_index()
+
+            fund_nav = fund_nav.loc[mdd_date1:mdd_date2].rename(columns=desc_name_dic)
+            fund_nav = fund_nav.reindex(pd.date_range(start=fund_nav.index[0], end=fund_nav.index[-1]))
+            fund_nav.index.name = 'datetime'
             data = {
                 'mdd': df_result,
                 'fund_nav': fund_nav,
@@ -600,17 +732,18 @@ class PortfolioDataApi(metaclass=Singleton):
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_port_fund_mdd_details')
 
-    def get_port_rolling_beta_index_info(self, fund_type):
-        l = []
-        l += [['hs300','沪深300'],['csi500','中证500']] 
-        return l
+    def get_port_rolling_beta_index_info(self):
+        return [['hs300','沪深300'],['csi500','中证500']] 
 
-    def get_port_fund_alpha_detail(self, trade_history, benchmark_id_info):
+    def get_port_fund_alpha_detail(self, trade_history, benchmark_id_info): 
         try:
             order_list, fund_type = self.trade_history_parse(trade_history)
             result = PortfolioDataApi().get_portfolio_nav(trade_history=trade_history)
             fund_nav = result['各基净值']
             fund_wgt = result['各基权重']
+            if fund_nav.empty():
+                return []
+            
             dts_list = [j for i in order_list for j in i.keys()] + [fund_nav.index[-1]]
             dts_list = [[i,dts_list[dts_list.index(i)+1]] for i in dts_list if i != dts_list[-1]]
 
@@ -619,9 +752,10 @@ class PortfolioDataApi(metaclass=Singleton):
                                                             end_date=fund_nav.index[-1],
                                                         )
             index_price = index_price.pivot_table(index='datetime',columns='index_id',values='close')
-            fund_info = BasicDataApi().get_fund_info(fund_list=fund_nav.columns.tolist())
+            fund_info = self.get_all_fund_info(fund_list=fund_nav.columns.tolist())
             fund_desc_dic = fund_info.set_index('fund_id')['desc_name'].to_dict()
-
+            index_info = BasicDataApi().get_index_info(index_list=list(benchmark_id_info.values()))
+            index_desc_dic = index_info.set_index('index_id')['desc_name'].to_dict()
             result = {}
             for fund_ti,_fund_list in fund_type.items():
                 _result = []
@@ -641,7 +775,7 @@ class PortfolioDataApi(metaclass=Singleton):
                         'begin_date':str(b_d),
                         'end_date':str(e_d),
                         'index_ret': int(index_rets_i * 10000),
-                        'index_name':index_id,
+                        'index_name':index_desc_dic[index_id],
                         'fund_details':[],
                     }
                     for _fund_id in type_fund_list:
@@ -657,13 +791,22 @@ class PortfolioDataApi(metaclass=Singleton):
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from portfolio.get_port_fund_alpha_detail')
 
-    def get_port_recent_ret_details(self, order_list, fund_type, index_id, time_para, begin_date, end_date, benchmark_id_info):
-        #try:
+    def get_port_recent_ret_details(self, trade_history, index_id, time_para, begin_date, end_date, benchmark_id_info):
+        try:
+            order_list, fund_type = self.trade_history_parse(trade_history)
             begin_date, end_date = RawDataApi().get_date_range(time_para, begin_date, end_date)
-            result = PortfolioDataApi().get_portfolio_nav(order_list=order_list, fund_type=fund_type)
+            result = PortfolioDataApi().get_portfolio_nav(trade_history=trade_history)
             fund_nav=result['净值']
+            if fund_nav.empty:
+                data = {
+                '组合收益':pd.DataFrame(),
+                '组合收益描述':[],
+                '资产价格比收益':pd.DataFrame(),
+                '资产价格比收益描述':[],
+                '个基近期收益':pd.DataFrame(),
+            }
             port_nav = PortfolioDataApi().get_portfolio_mv(fund_nav=fund_nav, index_id=index_id,begin_date=begin_date,end_date=end_date,time_para=time_para)
-            fund_info = BasicDataApi().get_fund_info(fund_list = result['各基净值'].columns.tolist())
+            fund_info = self.get_all_fund_info(fund_list = result['各基净值'].columns.tolist())
             fund_desc_dic = fund_info.set_index('fund_id')['desc_name'].to_dict()
             fund_type_dic = { vi: k for k, v in fund_type.items() for vi in v}
             cols = port_nav['data'].columns
@@ -687,9 +830,11 @@ class PortfolioDataApi(metaclass=Singleton):
             recent_ret = result['各基净值'].loc[begin_date:].dropna(axis=1)
             recent_ret = (recent_ret.iloc[-1] / recent_ret.iloc[0] - 1).to_frame()
             recent_ret.columns = ['基金涨跌']
+            # TODO 考虑换仓
             wgt_df = result['各基权重'].tail(1).T
             wgt_df.columns = ['权重']
             recent_ret = recent_ret.join(wgt_df)
+            recent_ret = recent_ret.dropna()
             recent_ret.loc[:,'基金收益'] = recent_ret['基金涨跌'] * recent_ret['权重']
             recent_ret['基金涨跌'] = recent_ret['基金涨跌'].map(lambda x : str(round(x * 100,2))+'%')
             recent_ret['权重'] = recent_ret['权重'].map(lambda x: str(round(100*x,2))) +'%'
@@ -700,6 +845,8 @@ class PortfolioDataApi(metaclass=Singleton):
             recent_ret.loc[:,'基金类型'] = recent_ret.index.map(fund_type_dic)
             recent_ret = recent_ret[['基金名','开始时间','结束时间','基金类型','基金涨跌','权重','基金收益']]
             result_3 = recent_ret
+            result_2 = result_2.reindex(pd.date_range(start=result_2.index[0], end=result_2.index[-1]))
+            result_2.index.name = 'datetime'
             data = {
                 '组合收益':result_1,
                 '组合收益描述':result_1_str,
@@ -708,6 +855,5 @@ class PortfolioDataApi(metaclass=Singleton):
                 '个基近期收益':result_3,
             }
             return data
-
-        # except Exception as e:
-        #     print(f'Failed to get data <err_msg> {e} from portfolio.get_port_recent_ret_details')
+        except Exception as e:
+            print(f'Failed to get data <err_msg> {e} from portfolio.get_port_recent_ret_details')
