@@ -334,7 +334,7 @@ class FundIndicatorProcessor(object):
 
             self.init(start_date=start_date, end_date=end_date)
 
-            self._update_begin_date = self._dm.dts.trading_days[self._dm.dts.trading_days.datetime<self.end_date] 
+            self._update_begin_date = self._dm.dts.trading_days[self._dm.dts.trading_days.datetime <= self.end_date] 
             update_dt_list = self._update_begin_date.datetime.values[-3:].tolist()
             self.fac_result = []
             for dt in update_dt_list:
