@@ -45,7 +45,7 @@ class FOFDataManagerLite():
         self._manually: Optional[pd.DataFrame] = None
         self._fof_nav: Optional[pd.DataFrame] = None
         self._fof_position: Optional[pd.DataFrame] = None
-        self._fof_investor_pos: Optional[pd.DataFrame] = None
+        # self._fof_investor_pos: Optional[pd.DataFrame] = None
         self._fof_position_details: Optional[pd.DataFrame] = None
         self._total_net_assets: float = None
         self._total_shares: float = None
@@ -199,6 +199,10 @@ class FOFDataManagerLite():
         # 客户赎回记录
         self._fof_redemp = fof_scale[fof_scale.event_type.isin([FOFTradeStatus.REDEEM, ])].set_index('datetime')
 
+        investor_div_carry = DerivedDataApi().get_hedge_fund_investor_div_carry(manager_id, [fof_id])
+        # TODO: 还得处理其他类型的交易
+        self._investor_div_carry = investor_div_carry[investor_div_carry.event_type.isin([FOFTradeStatus.DIVIDEND_VOLUME, ])].set_index('datetime')
+
         # trading_day_list = BasicDataApi().get_trading_day_list(start_date=self._start_date, end_date=datetime.datetime.now().date())
         # 将昨天作为end_date
         self._end_date = datetime.datetime.now().date() - datetime.timedelta(days=1)
@@ -320,8 +324,7 @@ class FOFDataManagerLite():
         administrative_fee_list = pd.Series(dtype='float64', name='administrative_fee')
         deposit_interest_list = pd.Series(dtype='float64', name='interest')
         positions_list = pd.Series(dtype='float64', name='position')
-        errors_to_be_fixed_list = pd.Series(dtype='float64', name='errors')
-        investor_pos_list = []
+        # investor_pos_list = []
         for date in self._date_list:
             total_amount = 0
             share_increased = 0
@@ -332,9 +335,9 @@ class FOFDataManagerLite():
                 pass
             else:
                 # 后边都是汇总信息了 所以在这里先加一下记录
-                to_append_to_investor_pos = scale_data.loc[:, ['fof_id', 'investor_id', 'purchase_amount', 'raising_interest', 'share_changed', 'event_type']]
-                to_append_to_investor_pos['purchase_amount'] = to_append_to_investor_pos.purchase_amount.add(to_append_to_investor_pos.raising_interest, fill_value=0)
-                investor_pos_list.append(to_append_to_investor_pos.rename(columns={'purchase_amount': 'amount'}))
+                # to_append_to_investor_pos = scale_data.loc[:, ['fof_id', 'investor_id', 'purchase_amount', 'raising_interest', 'share_changed', 'event_type']]
+                # to_append_to_investor_pos['purchase_amount'] = to_append_to_investor_pos.purchase_amount.add(to_append_to_investor_pos.raising_interest, fill_value=0)
+                # investor_pos_list.append(to_append_to_investor_pos.rename(columns={'purchase_amount': 'amount'}))
 
                 purchase_sum = scale_data.purchase_amount.sum() + scale_data.raising_interest.sum()
                 share_sum = scale_data.share_changed.sum()
@@ -342,12 +345,14 @@ class FOFDataManagerLite():
                 total_amount += purchase_sum
                 assert not pd.isnull(share_sum), '!!!'
                 share_increased += share_sum
+                share_increased = round(share_increased, 2)
 
             if other_assets_list.empty:
                 other_assets = 0
             else:
                 other_assets = other_assets_list.iat[-1]
             other_assets += total_amount
+            other_assets = round(other_assets, 2)
 
             try:
                 # 看看当天有没有卖出FOF
@@ -357,7 +362,7 @@ class FOFDataManagerLite():
                 total_amount_redemp = 0
             else:
                 # 后边都是汇总信息了 所以在这里先加一下记录
-                investor_pos_list.append(redemp_data.loc[:, ['fof_id', 'investor_id', 'redemp_confirmed_amount', 'share_changed', 'event_type']].rename(columns={'redemp_confirmed_amount': 'amount'}))
+                # investor_pos_list.append(redemp_data.loc[:, ['fof_id', 'investor_id', 'redemp_confirmed_amount', 'share_changed', 'event_type']].rename(columns={'redemp_confirmed_amount': 'amount'}))
 
                 # TODO 处理水位线
                 # 汇总今天所有的赎回资金
@@ -379,6 +384,19 @@ class FOFDataManagerLite():
                     shares_list.loc[date] = share_increased
                 continue
 
+            try:
+                # 看看当天FOF有没有分红
+                investor_div_data = self._investor_div_carry.loc[[date], :]
+            except KeyError:
+                pass
+            else:
+                share_increased += investor_div_data.share_changed.sum()
+                # 这里减掉现金分红和业绩报酬
+                other_assets -= (investor_div_data.cash_dividend.sum() + investor_div_data.carry_amount.sum())
+            finally:
+                share_increased = round(share_increased, 2)
+                other_assets = round(other_assets, 2)
+
             try:
                 expenses = 0
                 today_asset_alloc = asset_alloc.loc[[date], :]
@@ -448,7 +466,12 @@ class FOFDataManagerLite():
                 deposit_interest = round(other_assets_list.iat[-1] * self._DEPOSIT_INTEREST_PER_YEAR / self._DAYS_PER_YEAR_FOR_INTEREST, 2)
             deposit_interest_list.loc[date] = deposit_interest
 
-            other_assets += (deposit_interest - misc_fees)
+            # 计算修正净资产
+            try:
+                errors_to_be_fixed = round(self._manually.at[date, 'amount'], 2)
+            except KeyError:
+                errors_to_be_fixed = 0
+            other_assets += (deposit_interest - misc_fees + errors_to_be_fixed)
             other_assets_list.loc[date] = other_assets
 
             # 获取持仓中当日公募、私募基金的MV
@@ -502,13 +525,7 @@ class FOFDataManagerLite():
                 position = pd.Series().to_frame()
             positions_list.loc[date] = json.dumps(position.to_dict(orient='records'))
 
-            # 计算修正净资产
-            try:
-                errors_to_be_fixed = round(self._manually.at[date, 'amount'], 2)
-                errors_to_be_fixed_list.loc[date] = errors_to_be_fixed
-            except KeyError:
-                errors_to_be_fixed = 0
-            today_net_assets_fixed = today_net_assets + errors_to_be_fixed_list.sum()
+            today_net_assets_fixed = today_net_assets
             net_assets_fixed_list.loc[date] = today_net_assets_fixed
 
             # 如果今日有投资人申购fof 记录下来
@@ -540,10 +557,10 @@ class FOFDataManagerLite():
         self._fof_position['manager_id'] = manager_id
         self._fof_position['deposit_in_bank'] = None
 
-        self._fof_investor_pos = pd.concat(investor_pos_list).rename(columns={'share_changed': 'shares'}).reset_index(drop=True)
-        self._fof_investor_pos = self._fof_investor_pos.groupby(by=['fof_id', 'investor_id'], sort=False).sum().reset_index()
-        self._fof_investor_pos['datetime'] = date
-        self._fof_investor_pos['manager_id'] = manager_id
+        # self._fof_investor_pos = pd.concat(investor_pos_list).rename(columns={'share_changed': 'shares'}).reset_index(drop=True)
+        # self._fof_investor_pos = self._fof_investor_pos.groupby(by=['fof_id', 'investor_id'], sort=False).sum().reset_index()
+        # self._fof_investor_pos['datetime'] = date
+        # self._fof_investor_pos['manager_id'] = manager_id
 
         self._total_net_assets = net_assets_list.array[-1]
         self._total_shares = shares_list.sum()
@@ -567,6 +584,7 @@ class FOFDataManagerLite():
             else:
                 all_pos = None
         if all_pos is not None and self._all_latest_nav is not None:
+            all_pos = all_pos[all_pos != 0]
             latest_mv = all_pos * self._all_latest_nav
             dividend = self._asset_allocation.loc[self._asset_allocation.event_type.isin([FOFTradeStatus.DIVIDEND_CASH, FOFTradeStatus.DEDUCT_REWARD_AND_DIVIDEND_CASH]), :].pivot(index='datetime', columns='fund_id', values='amount').sum()
             redemptions = self._asset_allocation.loc[self._asset_allocation.event_type == FOFTradeStatus.REDEEM, :].pivot(index='datetime', columns='fund_id', values='share').sum()
@@ -588,6 +606,7 @@ class FOFDataManagerLite():
                                      self._hedge_latest_v_nav.rename('v_nav') if self._hedge_latest_v_nav is not None else pd.Series(name='v_nav'),
                                      latest_mv.rename('latest_mv') if latest_mv is not None else pd.Series(name='latest_mv')]
         self._fof_position_details = self._water_line_and_confirmed_nav.join(fof_position_details_list, how='outer')
+        self._fof_position_details = self._fof_position_details[self._fof_position_details.total_shares.notna()]
         self._fof_position_details = self._fof_position_details.rename_axis(index='fund_id').reset_index()
         self._fof_position_details['fof_id'] = fof_id
         self._fof_position_details['manager_id'] = manager_id
@@ -649,6 +668,7 @@ class FOFDataManagerLite():
             print('[dump_fof_nav_and_pos_to_db] no position, should calc it first')
 
         if self._fof_position_details is not None and not self._fof_position_details.empty:
+            print(self._fof_position_details)
             now_df = DerivedDataApi().get_fof_position_detail(manager_id, [fof_id])
             if now_df is not None:
                 now_df = now_df.drop(columns=['update_time', 'create_time', 'is_deleted']).astype(self._fof_position_details.dtypes.to_dict())
@@ -669,7 +689,6 @@ class FOFDataManagerLite():
         investor_return_df = FOFDataManagerLite().get_investor_return(manager_id=manager_id, fof_id=fof_id)
         if investor_return_df is not None:
             investor_return_df = investor_return_df.drop(columns=['total_rr']).rename(columns={'latest_mv': 'mv'})
-            investor_return_df['manager_id'] = manager_id
             now_df = FOFDataManagerLite().get_fof_investor_position(manager_id, [fof_id])
             if now_df is not None:
                 now_df = now_df.drop(columns=['update_time', 'create_time', 'is_deleted'])
@@ -684,6 +703,9 @@ class FOFDataManagerLite():
             if not df.empty:
                 BasicDataApi().delete_fof_investor_position(manager_id=manager_id, fof_id_to_delete=fof_id, investor_id_list=df[(df.manager_id == manager_id) & (df.fof_id == fof_id)].investor_id.to_list())
                 df.to_sql(FOFInvestorPosition.__table__.name, BasicDatabaseConnector().get_engine(), index=False, if_exists='append')
+            stale_investor_list = set(now_df.investor_id.array).difference(set(investor_return_df.index.array))
+            print(f'[dump_fof_nav_and_pos_to_db] (manager_id){manager_id} (fof_id){fof_id} (stale_investor_list){stale_investor_list}')
+            BasicDataApi().delete_fof_investor_position(manager_id=manager_id, fof_id_to_delete=fof_id, investor_id_list=stale_investor_list)
             print('[dump_fof_nav_and_pos_to_db] dump investor position done')
 
             investor_data = investor_return_df[['fof_id', 'amount', 'manager_id']]
@@ -1015,7 +1037,8 @@ class FOFDataManagerLite():
             return
         fof_nav = fof_nav[fof_nav.fof_id.isin(fof_scale_info.fof_id.unique())]
         fof_nav = fof_nav.pivot(index='datetime', columns='fof_id', values='nav')
-        fof_nav = fof_nav.reindex(fof_nav.index.union(fof_scale_info.datetime.unique())).rename_axis(index='datetime').sort_index().ffill()
+        fof_nav = fof_nav.reindex(fof_nav.index.union(fof_scale_info.datetime.unique())).rename_axis(index='datetime')
+        fof_nav = fof_nav.combine_first(fof_scale_info.drop_duplicates(subset=['datetime', 'fof_id'], keep='last').pivot(index='datetime', columns='fof_id', values='net_asset_value')).sort_index().ffill()
 
         investor_pos = {}
         for row in fof_scale_info.itertuples(index=False):
@@ -1070,10 +1093,6 @@ class FOFDataManagerLite():
         fof_nav = DerivedDataApi().get_fof_nav(manager_id, [fof_id])
         if fof_nav is None:
             return
-        investor_pur_redemp = DerivedDataApi().get_hedge_fund_investor_pur_redemp(manager_id, [fof_id])
-        if investor_pur_redemp is None:
-            return
-
         fof_nav = fof_nav.set_index('datetime')
         if start_date:
             fof_nav = fof_nav[fof_nav.index >= start_date]
@@ -1081,6 +1100,15 @@ class FOFDataManagerLite():
             fof_nav = fof_nav[fof_nav.index <= end_date]
         if fof_nav.empty:
             return
+
+        investor_pur_redemp = DerivedDataApi().get_hedge_fund_investor_pur_redemp(manager_id, [fof_id])
+        if investor_pur_redemp is None:
+            return
+        investor_pur_redemp = investor_pur_redemp.sort_values(by='datetime')
+        investor_div_carry = DerivedDataApi().get_hedge_fund_investor_div_carry(manager_id, [fof_id])
+        if investor_div_carry is None:
+            return
+        investor_div_carry = investor_div_carry.sort_values(by='datetime')
         investor_mvs = defaultdict(list)
         for row in investor_pur_redemp.itertuples(index=False):
             if investor_id_list and row.investor_id not in investor_id_list:
@@ -1101,16 +1129,30 @@ class FOFDataManagerLite():
             elif row.event_type in (FOFTradeStatus.REDEEM, ):
                 share_redemp = row.share_changed
                 for one in investor_mvs[row.investor_id]:
-                    if one['left_share'] >= share_redemp:
-                        one['left_share'] += share_redemp
+                    if one['left_share'] >= abs(share_redemp):
+                        one['left_share'] -= abs(share_redemp)
                         one['trade_ids'].append(row.id)
                         break
                     else:
-                        share_redemp -= one['left_share']
                         one['left_share'] = 0
                         one['left_amount'] = 0
                         one['trade_ids'].append(row.id)
 
+        for row in investor_div_carry.itertuples(index=False):
+            if investor_id_list and row.investor_id not in investor_id_list:
+                continue
+            # TODO: 需要处理其他几种类型的交易
+            if row.event_type in (FOFTradeStatus.DIVIDEND_VOLUME, ):
+                assert row.investor_id in investor_mvs, '!!!'
+                share_dividend = row.share_changed
+                is_added = False
+                for one in investor_mvs[row.investor_id]:
+                    if not is_added:
+                        one['left_share'] += share_dividend
+                        is_added = True
+                    # 水位线都改成当前的这个
+                    one['water_line'] = row.acc_unit_value
+
         investor_returns = {}
         latest_nav = fof_nav.nav.array[-1]
         for investor_id, datas in investor_mvs.items():
@@ -1120,12 +1162,11 @@ class FOFDataManagerLite():
             # 这里确保单取的几个Series的顺序不会发生任何变化 这样直接运算才是OK的
             mv = latest_nav * datas.left_share
             latest_mv = mv.sum()
-            total_mv = (latest_nav * datas.share).sum()
             total_share = datas.left_share.sum()
             avg_v_nav = latest_mv / total_share
             amount_sum = datas.amount.sum()
             if not math.isclose(amount_sum, 0):
-                total_ret = total_mv - amount_sum
+                total_ret = latest_mv - amount_sum
                 total_rr = total_ret / amount_sum
             else:
                 total_ret = np.nan
@@ -1135,7 +1176,7 @@ class FOFDataManagerLite():
             else:
                 avg_nav_cost = np.nan
             investor_returns[investor_id] = {
-                'datetime': fof_nav.index.array[-1], 'fof_id': fof_id, 'v_nav': avg_v_nav, 'cost_nav': avg_nav_cost, 'total_ret': total_ret, 'total_rr': total_rr, 'amount': amount_sum, 'shares': total_share, 'latest_mv': latest_mv,
+                'datetime': fof_nav.index.array[-1], 'manager_id': manager_id, 'fof_id': fof_id, 'v_nav': avg_v_nav, 'cost_nav': avg_nav_cost, 'total_ret': total_ret, 'total_rr': total_rr, 'amount': amount_sum, 'shares': total_share, 'latest_mv': latest_mv,
                 'details': datas[['trade_ids', 'amount', 'left_share', 'water_line']].rename(columns={'left_share': 'share'}).join(mv.rename('mv')).to_json(orient='records')
             }
         return pd.DataFrame.from_dict(investor_returns, orient='index').rename_axis(index='investor_id')
