@@ -185,6 +185,7 @@ class HedgeFundDataManager(metaclass=Singleton):
 
         s = pd.Series(datas).drop(['amount', 'redemp_share'], errors='ignore')
         # TODO: 原子操作下面几步
+        # TODO: 先刷后边的再删这一条
         if 'id' in datas:
             DerivedDataApi().delete_hedge_fund_investor_pur_redemp(id_to_delete=datas['id'])
         sql_obj = HedgeFundInvestorPurAndRedemp()
@@ -274,16 +275,19 @@ class HedgeFundDataManager(metaclass=Singleton):
         else:
             total_share, share_separate = self._get_total_share(datas['manager_id'], datas['fof_id'], datas['investor_id'])
             share_separate = share_separate.share_changed.array[-1]
-            div = datas['total_dividend'] * share_separate / total_share
+            # div = datas['total_dividend'] * share_separate / total_share
 
+            div = datas['total_dividend']
             if event_type == FOFTradeStatus.DIVIDEND_CASH:
                 # 现金分红
                 datas['share_changed'] = 0
-                datas['cash_dividend'] = div
+                datas['cash_dividend'] = div - (datas['carry_amount'] if not pd.isnull(datas['carry_amount']) else 0)
+                datas['reinvest_amount'] = 0
             elif event_type == FOFTradeStatus.DIVIDEND_VOLUME:
                 # 红利再投
-                datas['share_changed'] = (div - (datas['carry_amount'] if not pd.isnull(datas['carry_amount']) else 0)) / datas['net_asset_value']
-                datas['reinvest_amount'] = div
+                datas['reinvest_amount'] = div - (datas['carry_amount'] if not pd.isnull(datas['carry_amount']) else 0)
+                datas['share_changed'] = datas['reinvest_amount'] / datas['net_asset_value']
+                datas['cash_dividend'] = 0
             else:
                 raise ValueError(f'invalid event_type {event_type}')
 
@@ -360,7 +364,8 @@ class HedgeFundDataManager(metaclass=Singleton):
         event_type = datas['event_type']
         if datas['event_type'] == FOFTradeStatus.DIVIDEND_CASH:
             # 现金分红
-            df = s.to_frame('cash_dividend')
+            df = s.to_frame('total_dividend')
+            df['cash_dividend'] = df.total_dividend
             if 'do_deduct_reward' in datas and datas['do_deduct_reward']:
                 carry_df = self.calc_carry_event(datas)
                 df = df.join(carry_df.set_index('investor_id').carry_amount)
@@ -371,7 +376,8 @@ class HedgeFundDataManager(metaclass=Singleton):
             df['reinvest_amount'] = 0
         elif datas['event_type'] == FOFTradeStatus.DIVIDEND_VOLUME:
             # 红利再投
-            df = s.to_frame('reinvest_amount')
+            df = s.to_frame('total_dividend')
+            df['reinvest_amount'] = df.total_dividend
             if 'do_deduct_reward' in datas and datas['do_deduct_reward']:
                 carry_df = self.calc_carry_event(datas)
                 df = df.join(carry_df.set_index('investor_id').carry_amount)
@@ -388,7 +394,6 @@ class HedgeFundDataManager(metaclass=Singleton):
         df['manager_id'] = datas['manager_id']
         df['datetime'] = datas['datetime']
         df['event_type'] = datas['event_type']
-        df['total_dividend'] = dividend_amount
         return df.reset_index()
 
     def calc_carry_event(self, datas: Dict[str, Any]) -> pd.DataFrame:
