@@ -705,7 +705,7 @@ class FOFNav(FOFBase):
     __tablename__ = 'fof_nav'
 
     fof_id = Column(CHAR(16), primary_key=True)             # 产品ID
-    manager_id = Column(CHAR(32))                           # 管理端ID
+    manager_id = Column(CHAR(32), primary_key=True)         # 管理端ID
     datetime = Column(DATE, primary_key=True)               # 日期
     nav = Column(DOUBLE(asdecimal=False), nullable=False)   # 单位净值
     acc_net_value = Column(DOUBLE(asdecimal=False))         # 累计净值
@@ -714,7 +714,18 @@ class FOFNav(FOFBase):
     volume = Column(DOUBLE(asdecimal=False))                # 份额
     mv = Column(DOUBLE(asdecimal=False))                    # 市值
     ret = Column(DOUBLE(asdecimal=False))                   # 累计收益率
-    manager_id = Column(CHAR(32))                           # 管理端ID
+
+
+class FOFUnconfirmedNav(FOFBase):
+    '''fof未确认净值数据'''
+
+    __tablename__ = 'fof_nav_unconfirmed'
+
+    fof_id = Column(CHAR(16), primary_key=True)             # 产品ID
+    manager_id = Column(CHAR(32), primary_key=True)         # 管理端ID
+    datetime = Column(DATE, primary_key=True)               # 日期
+    nav = Column(DOUBLE(asdecimal=False), nullable=False)   # 单位净值
+    acc_net_value = Column(DOUBLE(asdecimal=False))         # 累计净值
 
 
 class FOFNavPublic(FOFBase):
