@@ -1227,8 +1227,10 @@ class Calculator:
 
             result = Calculator.get_rolling_corr(fund_navs, windows, period)
             if result.empty:
+                df = pandas.DataFrame()
+                df.index.name = 'datetime'
                 return {
-                    'data':result,
+                    'data':df,
                     'details':{},
                 }
             if result.shape[0] > 50:
@@ -1260,6 +1262,13 @@ class Calculator:
                 df_dt.columns.name = None
                 df_dt.index.name = 'index_id'
                 corr_details[str(r[0])] = df_dt
+            if df_dt.empty:
+                df = pandas.DataFrame()
+                df.index.name = 'datetime'
+                return {
+                'data':df,
+                'details':corr_details
+            }
             return {
                 'data':result,
                 'details':corr_details
@@ -1267,14 +1276,18 @@ class Calculator:
 
         except Exception as e:
             print(f'Failed to get data <err_msg> {e} from calculator.get_product_rolling_corr')
+            df = pandas.DataFrame()
+            df.index.name = 'datetime'
             return {
-                    'data':result,
+                    'data':df,
                     'details':{},
                 }
 
     @staticmethod
     def get_product_rolling_beta(fund_navs, windows=20, period='周度', index_id='hs300'):
         try:
+            if fund_navs.empty:
+                return pandas.DataFrame()
             if period == '日度':
                 rule = '1D'
                 if windows <= 2:
