@@ -397,7 +397,10 @@ class CalculatorBase:
             cumu_ret = last_unit_nav - 1
             trade_year = days / CalculatorBase.TOTAL_DAYS_PER_YEAR
             annual_ret = math.pow(last_unit_nav, CalculatorBase.TOTAL_DAYS_PER_YEAR/days) - 1
-            annual_vol = sr_ret_annual_vol.std(ddof=1) * np.sqrt(period_1y)
+            if frequency == '1D':
+                annual_vol = sr_ret_annual_vol.std(ddof=1) * np.sqrt(period_1y)
+            else:
+                annual_vol = sr.std(ddof=1) * np.sqrt(period_1y)
             _begin_date = sr.index[0]
             recent_ret_res = CalculatorBase.calc_recent_ret(dates=dates,values=values,frequency='1D')
             recent_year_ret = recent_ret_res['recent_natural_year_ret']
