@@ -104,21 +104,25 @@ class MailAttachmentRetriever:
                                     if pos_start != -1:
                                         end_tag = b'</table'
                                         pos_end = body.find(end_tag)
-                                        df = pd.read_html(body[pos_start:pos_end+len(end_tag)], encoding='gbk')
-                                        if df:
-                                            df = df[0]
-                                            subject = str(make_header(decode_header(email_message['Subject'])))
-                                            fund_id = re.findall(r'S[A-Z0-9]{5}', subject, flags=re.ASCII)
-                                            if fund_id and '产品代码' not in df.columns.array:
-                                                df['产品代码'] = fund_id[0]
-                                            real_name = f"{subject}.xlsx"
-                                            file_path = os.path.join(self._dump_dir, real_name)
-                                            try:
-                                                df.to_excel(file_path, index=False)
-                                                df_list[real_name] = (uid, os.path.abspath(file_path))
-                                                print(f'content {real_name} on {df_list[real_name]} done (uid){uid}')
-                                            except Exception as e:
-                                                pass
+                                        try:
+                                            df = pd.read_html(body[pos_start:pos_end+len(end_tag)], encoding='gbk')
+                                        except Exception as e:
+                                            print(f'got exception when read html (exception){e}')
+                                        else:
+                                            if df:
+                                                df = df[0]
+                                                subject = str(make_header(decode_header(email_message['Subject'])))
+                                                fund_id = re.findall(r'S[A-Z0-9]{5}', subject, flags=re.ASCII)
+                                                if fund_id and '产品代码' not in df.columns.array:
+                                                    df['产品代码'] = fund_id[0]
+                                                real_name = f"{subject}.xlsx"
+                                                file_path = os.path.join(self._dump_dir, real_name)
+                                                try:
+                                                    df.to_excel(file_path, index=False)
+                                                    df_list[real_name] = (uid, os.path.abspath(file_path))
+                                                    print(f'content {real_name} on {df_list[real_name]} done (uid){uid}')
+                                                except Exception as e:
+                                                    pass
                 M.close()
             return df_list
         except Exception as e:
